 // --------------------- \\
// PC Mod - For Server \\
 // --------------------- \\

// Define the PCMod Core
PCMod = {}
PCMod.Exists = true
PCMod.Version = 12
PCMod.Routers = {}

// Tell the console we are loading
Msg( "=== PCMod Installed ===\n" )
Msg( "== PCMod Serverside OS (BETA Version " .. PCMod.Version .. ") Initialized ==\n" )

// Add the serverside pathfinding module
include( "sv_pcmod_pathfinding.lua" )

// Add the clientside main file, plus other stuff
AddCSLuaFile( "autorun/client/cl_pcmod.lua" )


AddCSLuaFile( "cl_themes.lua" )
AddCSLuaFile( "themes/th_default-blue.lua" )
AddCSLuaFile( "themes/th_default-red.lua" )

// Uses DMenu LUA Module
AddCSLuaFile( "autorun/client/cl_dmenu.lua" )

// Here we will load all our programs
include( "sv_programs.lua" )

// Some globals
PData = {}
PCMod.TimedMessage = true
// ^ Set this to false to stop the message that comes up every 90 seconds.
PCMod.CustomTags = false
// ^ Set this to true to change sv_tags to 'PCMod'
if (!WireAddon) then WireAddon = 0 end
// ^ Don't change
PCMod.WirelessRange = 350
// ^ Range of a wireless connection in units

// Network Pathfinding
PacketPath = {}

function Add_VGUI_Mat( mat )
	resource.AddFile( "materials/vgui/" .. mat .. ".vmt" )
	resource.AddFile( "materials/vgui/" .. mat .. ".vtf" )
end

// Here we will initialize the addon
function InitAddon()
	// Add our resource files
	Add_VGUI_Mat( "pc_screen1" )
	Add_VGUI_Mat( "pc_screen2" )
	Add_VGUI_Mat( "pc_screen3" )
	
	Add_VGUI_Mat( "glossyblue" )
	Add_VGUI_Mat( "glossyblue-right" )
	Add_VGUI_Mat( "glossyblue-left" )
	Add_VGUI_Mat( "glossyblue-middle" )
	
	Add_VGUI_Mat( "glossywhite" )
	Add_VGUI_Mat( "glossywhite-right" )
	Add_VGUI_Mat( "glossywhite-left" )
	Add_VGUI_Mat( "glossywhite-middle" )
	
	Add_VGUI_Mat( "lowerbar-right" )
	
	Add_VGUI_Mat( "connect" )
	Add_VGUI_Mat( "disconnect" )
	Add_VGUI_Mat( "transmit" )
	Add_VGUI_Mat( "transmit_blue" )
	
	// Laptop Model
	resource.AddFile( "models/laptop/laptop.mdl" )
	resource.AddFile( "models/laptop/laptop.phy" )
	resource.AddFile( "models/laptop/laptop.vvd" )
	resource.AddFile( "models/laptop/laptop.sw.vtx" )
	resource.AddFile( "models/laptop/laptop.dx80.vtx" )
	resource.AddFile( "models/laptop/laptop.dx90.vtx" )
	
	// Laptop Material
	resource.AddFile( "materials/models/laptop/laptop.vtf" )
	resource.AddFile( "materials/models/laptop/laptop_normal.vtf" )
	resource.AddFile( "materials/models/laptop/laptop.vmt" )
	
	// Setup a timer that prints a message to everybody every 90 seconds
	if (PCMod.TimedMessage) then
		timer.Create( "PCMod_HintTimer", 90, 0, function()
			for k,v in pairs( player.GetAll() ) do
				v:PrintMessage( HUD_PRINTTALK, "This server uses PCMod Addon BETA V" .. PCMod.Version )
			end
		end )
	end
	
	// If we are told to, add custom tags
	if (PCMod.CustomTags) then game.ConsoleCommand( "sv_tags PCMod\n" ) end
end
hook.Add( "Initialize", "PCMod_InitAddon", InitAddon )

// Simple function that pulls the subnet out of an IP
function GetSubnet( IP )
	return (string.Explode( ".", IP )[3])
end

// This allows general PC commands to be called
function CC_PC( ply, command, args )
	// Check if the player is using a PC
	if (!ply.CurrentPC) then return end
	if (!ply.CurrentPC:IsValid()) then return end
	local obj = ply.CurrentPC
	
	// Get the command, then remove it from the table
	pc_com = args[1]
	table.remove( args, 1 )
	
	// Call the command
	obj:ExecuteCommand( ply, pc_com, args )
end
concommand.Add( "pc_command", CC_PC )

// DeliveryCode:
// 0 = Cannot Deliver
// 1 = Delivered
// 2 = Destination not found
// 3 = No Network
// 4 = Packet Denied: Closed Port
// 5 = Packet Denied: Blocked IP
// 6 = Packet Denied: Invalid packet

function DeliveryStatus( response )
	if (response == 0) then return "Cannot Deliver" end
	if (response == 1) then return "Packet Delivered" end
	if (response == 2) then return "Destination Not Found" end
	if (response == 3) then return "No Network Availible" end
	if (response == 4) then return "Packet Denied: Closed Port" end
	if (response == 5) then return "Packet Denied: IP Blocked" end
	if (response == 6) then return "Packet Denied: Invalid Packet" end
	return "Unknown Status"
end

function PCMOD_Spawn( ply )
	umsg.Start( "_haswire", ply )
	umsg.Short( WireAddon )
	umsg.End()
end
hook.Add( "PlayerSpawn", "PCMod_Spawn", PCMOD_Spawn )

// This is called by the current program and the OS. Returns: DeliveryCode
function ComposePacket( ply, dest, port, body, pass )
	local obj = ply.CurrentPC
	if (!obj) then return 0 end
	packet = { obj.Data[obj:EntIndex()].IP, dest, port, body }
	if (pass) then packet[5] = pass end
	return obj:SendPacket( packet )
end

// These are called by the current program and the OS. Returns: Nothing.
function OpenPort( ply, port )
	local obj = ply.CurrentPC
	if (!obj) then return end
	table.insert( obj.Data[obj:EntIndex()].PortList, { port, "prog" } )
end
function ClosePort( ply, port )
	local obj = ply.CurrentPC
	if (!obj) then return end
end

// These are called by the current program. Returns: Nothing.
function SendClientProgram( ply, ProgramName )
	if (!ply) then return end
	umsg.Start( "_pcmod_runprog", ply )
	umsg.String( ProgramName )
	umsg.End()
end
function SendClientWindow( ply, WindowName )
	if (!ply) then return end
	umsg.Start( "_pcmod_progwindow", ply )
	umsg.String( WindowName )
	umsg.End()
	local obj = ply.CurrentPC
	if (!obj) then return end
	obj.Data[obj:EntIndex()].WindowName = WindowName
end
function UnloadProgram( ply )
	if (!ply) then return end
	umsg.Start( "_pcmod_killprog", ply )
	umsg.End()
	local obj = ply.CurrentPC
	if (!obj) then return end
	obj:UnloadProgram()
end
function BeamData_String( ply, tag, data )
	if (!ply) then return end
	umsg.Start( "_pcmod_beamdata_str", ply )
		umsg.String( tag )
		umsg.String( data )
	umsg.End()
end
function BeamData_Int( ply, tag, data )
	if (!ply) then return end
	umsg.Start( "_pcmod_beamdata_int", ply )
		umsg.String( tag )
		umsg.Short( data )
	umsg.End()
end
function BeamData_Float( ply, tag, data )
	if (!ply) then return end
	umsg.Start( "_pcmod_beamdata_float", ply )
		umsg.String( tag )
		umsg.Float( data )
	umsg.End()
end

// These functions set and get networked OS vars
function SetOSVar( pc, VarName, VarValue )
	local tp = type( VarValue )
	local vn = "OS_" .. VarName
	if (!VarValue) then return end
	if (tp == "boolean") then pc:SetNWBool( vn, VarValue ) end
	if (tp == "string") then pc:SetNWString( vn, VarValue ) end
	if (tp == "number") then pc:SetNWInt( vn, VarValue ) end
end
function GetOSVar( pc, VarType, VarName )
	local vn = "OS_" .. VarName
	if (tp == "boolean") then return pc:GetNWBool( vn ) end
	if (tp == "string") then return pc:GetNWString( vn ) end
	if (tp == "number") then return pc:GetNWInt( vn ) end
end

Msg( "== PCMod Serverside OS Loaded ==\n" )